@@ -1,9 +1,11 @@
 from collections import namedtuple
 from six import string_types
+import re
 
 from samtranslator.model.intrinsics import ref
 from samtranslator.model.apigateway import (ApiGatewayDeployment, ApiGatewayRestApi,
-                                            ApiGatewayStage, ApiGatewayAuthorizer)
+                                            ApiGatewayStage, ApiGatewayAuthorizer,
+                                            ApiGatewayResponse)
 from samtranslator.model.exceptions import InvalidResourceException
 from samtranslator.model.s3_utils.uri_parser import parse_s3_uri
 from samtranslator.region_configuration import RegionConfiguration
@@ -18,8 +20,11 @@ CorsProperties = namedtuple("_CorsProperties", ["AllowMethods", "AllowHeaders",
 # Default the Cors Properties to '*' wildcard and False AllowCredentials. Other properties are actually Optional
 CorsProperties.__new__.__defaults__ = (None, None, _CORS_WILDCARD, None, False)
 
-AuthProperties = namedtuple("_AuthProperties", ["Authorizers", "DefaultAuthorizer"])
-AuthProperties.__new__.__defaults__ = (None, None)
+AuthProperties = namedtuple("_AuthProperties",
+                            ["Authorizers", "DefaultAuthorizer", "InvokeRole", "AddDefaultAuthorizerToCorsPreflight"])
+AuthProperties.__new__.__defaults__ = (None, None, None, True)
+
+GatewayResponseProperties = ["ResponseParameters", "ResponseTemplates", "StatusCode"]
 
 
 class ApiGenerator(object):
@@ -27,8 +32,9 @@ class ApiGenerator(object):
     def __init__(self, logical_id, cache_cluster_enabled, cache_cluster_size, variables, depends_on,
                  definition_body, definition_uri, name, stage_name, endpoint_configuration=None,
                  method_settings=None, binary_media=None, minimum_compression_size=None, cors=None,
-                 auth=None, access_log_setting=None, canary_setting=None, tracing_enabled=None,
-                 resource_attributes=None, passthrough_resource_attributes=None):
+                 auth=None, gateway_responses=None, access_log_setting=None, canary_setting=None,
+                 tracing_enabled=None, resource_attributes=None, passthrough_resource_attributes=None,
+                 open_api_version=None, models=None):
         """Constructs an API Generator class that generates API Gateway resources
 
         :param logical_id: Logical id of the SAM API Resource
@@ -45,6 +51,7 @@ class ApiGenerator(object):
         :param tracing_enabled: Whether active tracing with X-ray is enabled
         :param resource_attributes: Resource attributes to add to API resources
         :param passthrough_resource_attributes: Attributes such as `Condition` that are added to derived resources
+        :param models: Model definitions to be used by API methods
         """
         self.logical_id = logical_id
         self.cache_cluster_enabled = cache_cluster_enabled
@@ -61,11 +68,14 @@ class ApiGenerator(object):
         self.minimum_compression_size = minimum_compression_size
         self.cors = cors
         self.auth = auth
+        self.gateway_responses = gateway_responses
         self.access_log_setting = access_log_setting
         self.canary_setting = canary_setting
         self.tracing_enabled = tracing_enabled
         self.resource_attributes = resource_attributes
         self.passthrough_resource_attributes = passthrough_resource_attributes
+        self.open_api_version = open_api_version
+        self.models = models
 
     def _construct_rest_api(self):
         """Constructs and returns the ApiGateway RestApi.
@@ -74,6 +84,8 @@ class ApiGenerator(object):
         :rtype: model.apigateway.ApiGatewayRestApi
         """
         rest_api = ApiGatewayRestApi(self.logical_id, depends_on=self.depends_on, attributes=self.resource_attributes)
+        # NOTE: For backwards compatibility we need to retain BinaryMediaTypes on the CloudFormation Property
+        # Removing this and only setting x-amazon-apigateway-binary-media-types results in other issues.
         rest_api.BinaryMediaTypes = self.binary_media
         rest_api.MinimumCompressionSize = self.minimum_compression_size
 
@@ -89,8 +101,16 @@ class ApiGenerator(object):
             raise InvalidResourceException(self.logical_id,
                                            "Specify either 'DefinitionUri' or 'DefinitionBody' property and not both")
 
+        if self.open_api_version:
+            if re.match(SwaggerEditor.get_openapi_versions_supported_regex(), self.open_api_version) is None:
+                raise InvalidResourceException(
+                    self.logical_id, "The OpenApiVersion value must be of the format 3.0.0")
+
         self._add_cors()
         self._add_auth()
+        self._add_gateway_responses()
+        self._add_binary_media_types()
+        self._add_models()
 
         if self.definition_uri:
             rest_api.BodyS3Location = self._construct_body_s3_dict()
@@ -132,7 +152,7 @@ class ApiGenerator(object):
             body_s3['Version'] = s3_pointer['Version']
         return body_s3
 
-    def _construct_deployment(self, rest_api):
+    def _construct_deployment(self, rest_api, open_api_version):
         """Constructs and returns the ApiGateway Deployment.
 
         :param model.apigateway.ApiGatewayRestApi rest_api: the RestApi for this Deployment
@@ -142,7 +162,8 @@ class ApiGenerator(object):
         deployment = ApiGatewayDeployment(self.logical_id + 'Deployment',
                                           attributes=self.passthrough_resource_attributes)
         deployment.RestApiId = rest_api.get_runtime_attr('rest_api_id')
-        deployment.StageName = 'Stage'
+        if not self.open_api_version:
+            deployment.StageName = 'Stage'
 
         return deployment
 
@@ -184,7 +205,7 @@ class ApiGenerator(object):
         """
 
         rest_api = self._construct_rest_api()
-        deployment = self._construct_deployment(rest_api)
+        deployment = self._construct_deployment(rest_api, self.open_api_version)
 
         swagger = None
         if rest_api.Body is not None:
@@ -243,6 +264,24 @@ class ApiGenerator(object):
         # Assign the Swagger back to template
         self.definition_body = editor.swagger
 
+    def _add_binary_media_types(self):
+        """
+        Add binary media types to Swagger
+        """
+
+        if not self.binary_media:
+            return
+
+        # We don't raise an error here like we do for similar cases because that would be backwards incompatible
+        if self.binary_media and not self.definition_body:
+            return
+
+        editor = SwaggerEditor(self.definition_body)
+        editor.add_binary_media_types(self.binary_media)
+
+        # Assign the Swagger back to template
+        self.definition_body = editor.swagger
+
     def _add_auth(self):
         """
         Add Auth configuration to the Swagger file, if necessary
@@ -266,25 +305,138 @@ class ApiGenerator(object):
                                                             "'DefinitionBody' does not contain a valid Swagger")
         swagger_editor = SwaggerEditor(self.definition_body)
         auth_properties = AuthProperties(**self.auth)
-        authorizers = self._get_authorizers(auth_properties.Authorizers)
+        authorizers = self._get_authorizers(auth_properties.Authorizers, auth_properties.DefaultAuthorizer)
 
         if authorizers:
             swagger_editor.add_authorizers(authorizers)
-            self._set_default_authorizer(swagger_editor, authorizers, auth_properties.DefaultAuthorizer)
+            self._set_default_authorizer(swagger_editor, authorizers, auth_properties.DefaultAuthorizer,
+                                         auth_properties.AddDefaultAuthorizerToCorsPreflight)
+
+        # Assign the Swagger back to template
+
+        self.definition_body = self._openapi_postprocess(swagger_editor.swagger)
+
+    def _add_gateway_responses(self):
+        """
+        Add Gateway Response configuration to the Swagger file, if necessary
+        """
+
+        if not self.gateway_responses:
+            return
+
+        if self.gateway_responses and not self.definition_body:
+            raise InvalidResourceException(
+                self.logical_id, "GatewayResponses works only with inline Swagger specified in "
+                                 "'DefinitionBody' property")
+
+        # Make sure keys in the dict are recognized
+        for responses_key, responses_value in self.gateway_responses.items():
+            for response_key in responses_value.keys():
+                if response_key not in GatewayResponseProperties:
+                    raise InvalidResourceException(
+                        self.logical_id,
+                        "Invalid property '{}' in 'GatewayResponses' property '{}'".format(response_key, responses_key))
+
+        if not SwaggerEditor.is_valid(self.definition_body):
+            raise InvalidResourceException(
+                self.logical_id, "Unable to add Auth configuration because "
+                                 "'DefinitionBody' does not contain a valid Swagger")
+
+        swagger_editor = SwaggerEditor(self.definition_body)
+
+        gateway_responses = {}
+        for response_type, response in self.gateway_responses.items():
+            gateway_responses[response_type] = ApiGatewayResponse(
+                api_logical_id=self.logical_id,
+                response_parameters=response.get('ResponseParameters', {}),
+                response_templates=response.get('ResponseTemplates', {}),
+                status_code=response.get('StatusCode', None)
+            )
+
+        if gateway_responses:
+            swagger_editor.add_gateway_responses(gateway_responses)
 
         # Assign the Swagger back to template
         self.definition_body = swagger_editor.swagger
 
-    def _get_authorizers(self, authorizers_config):
+    def _add_models(self):
+        """
+        Add Model definitions to the Swagger file, if necessary
+        :return:
+        """
+
+        if not self.models:
+            return
+
+        if self.models and not self.definition_body:
+            raise InvalidResourceException(self.logical_id,
+                                           "Models works only with inline Swagger specified in "
+                                           "'DefinitionBody' property")
+
+        if not SwaggerEditor.is_valid(self.definition_body):
+            raise InvalidResourceException(self.logical_id, "Unable to add Models definitions because "
+                                                            "'DefinitionBody' does not contain a valid Swagger")
+
+        if not all(isinstance(model, dict) for model in self.models.values()):
+            raise InvalidResourceException(self.logical_id, "Invalid value for 'Models' property")
+
+        swagger_editor = SwaggerEditor(self.definition_body)
+        swagger_editor.add_models(self.models)
+
+        # Assign the Swagger back to template
+
+        self.definition_body = self._openapi_postprocess(swagger_editor.swagger)
+
+    def _openapi_postprocess(self, definition_body):
+        """
+        Convert definitions to openapi 3 in definition body if OpenApiVersion flag is specified.
+
+        If the is swagger defined in the definition body, we treat it as a swagger spec and dod not
+        make any openapi 3 changes to it
+        """
+        if definition_body.get('swagger') is not None:
+            return definition_body
+
+        if definition_body.get('openapi') is not None:
+            if self.open_api_version is None:
+                self.open_api_version = definition_body.get('openapi')
+
+        if self.open_api_version and re.match(SwaggerEditor.get_openapi_version_3_regex(), self.open_api_version):
+            if definition_body.get('securityDefinitions'):
+                components = definition_body.get('components', {})
+                components['securitySchemes'] = definition_body['securityDefinitions']
+                definition_body['components'] = components
+                del definition_body['securityDefinitions']
+            if definition_body.get('definitions'):
+                components = definition_body.get('components', {})
+                components['schemas'] = definition_body['definitions']
+                definition_body['components'] = components
+                del definition_body['definitions']
+        return definition_body
+
+    def _get_authorizers(self, authorizers_config, default_authorizer=None):
+        authorizers = {}
+        if default_authorizer == 'AWS_IAM':
+            authorizers[default_authorizer] = ApiGatewayAuthorizer(
+                api_logical_id=self.logical_id,
+                name=default_authorizer,
+                is_aws_iam_authorizer=True
+            )
+
         if not authorizers_config:
+            if 'AWS_IAM' in authorizers:
+                return authorizers
             return None
 
         if not isinstance(authorizers_config, dict):
             raise InvalidResourceException(self.logical_id,
                                            "Authorizers must be a dictionary")
-        authorizers = {}
 
         for authorizer_name, authorizer in authorizers_config.items():
+            if not isinstance(authorizer, dict):
+                raise InvalidResourceException(self.logical_id,
+                                               "Authorizer %s must be a dictionary." % (authorizer_name))
+
             authorizers[authorizer_name] = ApiGatewayAuthorizer(
                 api_logical_id=self.logical_id,
                 name=authorizer_name,
@@ -294,7 +446,6 @@ class ApiGenerator(object):
                 function_payload_type=authorizer.get('FunctionPayloadType'),
                 function_invoke_role=authorizer.get('FunctionInvokeRole')
             )
-
         return authorizers
 
     def _get_permission(self, authorizer_name, authorizer_lambda_function_arn):
@@ -342,16 +493,18 @@ class ApiGenerator(object):
 
         return permissions
 
-    def _set_default_authorizer(self, swagger_editor, authorizers, default_authorizer):
+    def _set_default_authorizer(self, swagger_editor, authorizers, default_authorizer,
+                                add_default_auth_to_preflight=True):
         if not default_authorizer:
             return
 
-        if not authorizers.get(default_authorizer):
+        if not authorizers.get(default_authorizer) and default_authorizer != 'AWS_IAM':
             raise InvalidResourceException(self.logical_id, "Unable to set DefaultAuthorizer because '" +
                                            default_authorizer + "' was not defined in 'Authorizers'")
 
         for path in swagger_editor.iter_on_path():
-            swagger_editor.set_path_default_authorizer(path, default_authorizer, authorizers=authorizers)
+            swagger_editor.set_path_default_authorizer(path, default_authorizer, authorizers=authorizers,
+                                                       add_default_auth_to_preflight=add_default_auth_to_preflight)
 
     def _set_endpoint_configuration(self, rest_api, value):
         """
