@@ -12,7 +12,7 @@ from samtranslator.model.lambda_ import LambdaPermission
 from samtranslator.model.events import EventsRule
 from samtranslator.model.iot import IotTopicRule
 from samtranslator.translator.arn_generator import ArnGenerator
-from samtranslator.model.exceptions import InvalidEventException
+from samtranslator.model.exceptions import InvalidEventException, InvalidResourceException
 from samtranslator.swagger.swagger import SwaggerEditor
 
 CONDITION = 'Condition'
@@ -386,7 +386,8 @@ class Api(PushEventSource):
 
             # Api Event sources must "always" be paired with a Serverless::Api
             'RestApiId': PropertyType(True, is_str()),
-            'Auth': PropertyType(False, is_type(dict))
+            'Auth': PropertyType(False, is_type(dict)),
+            'RequestModel': PropertyType(False, is_type(dict))
     }
 
     def resources_to_link(self, resources):
@@ -419,6 +420,8 @@ class Api(PushEventSource):
 
                 # Stage could be a intrinsic, in which case leave the suffix to default value
                 if isinstance(permitted_stage, string_types):
+                    if not permitted_stage:
+                        raise InvalidResourceException(rest_api_id, 'StageName cannot be empty.')
                     stage_suffix = permitted_stage
                 else:
                     stage_suffix = "Stage"
@@ -529,7 +532,7 @@ class Api(PushEventSource):
         if CONDITION in function.resource_attributes:
             condition = function.resource_attributes[CONDITION]
 
-        editor.add_lambda_integration(self.Path, self.Method, uri, condition=condition)
+        editor.add_lambda_integration(self.Path, self.Method, uri, self.Auth, api.get('Auth'), condition=condition)
 
         if self.Auth:
             method_authorizer = self.Auth.get('Authorizer')
@@ -538,28 +541,51 @@ class Api(PushEventSource):
                 api_auth = api.get('Auth')
                 api_authorizers = api_auth and api_auth.get('Authorizers')
 
-                if not api_authorizers:
+                if method_authorizer != 'AWS_IAM':
+                    if not api_authorizers:
+                        raise InvalidEventException(
+                            self.relative_id,
+                            'Unable to set Authorizer [{authorizer}] on API method [{method}] for path [{path}] '
+                            'because the related API does not define any Authorizers.'.format(
+                                authorizer=method_authorizer, method=self.Method, path=self.Path))
+
+                    if method_authorizer != 'NONE' and not api_authorizers.get(method_authorizer):
+                        raise InvalidEventException(
+                            self.relative_id,
+                            'Unable to set Authorizer [{authorizer}] on API method [{method}] for path [{path}] '
+                            'because it wasn\'t defined in the API\'s Authorizers.'.format(
+                                authorizer=method_authorizer, method=self.Method, path=self.Path))
+
+                    if method_authorizer == 'NONE' and not api_auth.get('DefaultAuthorizer'):
+                        raise InvalidEventException(
+                            self.relative_id,
+                            'Unable to set Authorizer on API method [{method}] for path [{path}] because \'NONE\' '
+                            'is only a valid value when a DefaultAuthorizer on the API is specified.'.format(
+                                method=self.Method, path=self.Path))
+
+                editor.add_auth_to_method(api=api, path=self.Path, method_name=self.Method, auth=self.Auth)
+
+        if self.RequestModel:
+            method_model = self.RequestModel.get('Model')
+
+            if method_model:
+                api_models = api.get('Models')
+                if not api_models:
                     raise InvalidEventException(
                         self.relative_id,
-                        'Unable to set Authorizer [{authorizer}] on API method [{method}] for path [{path}] because '
-                        'the related API does not define any Authorizers.'.format(
-                            authorizer=method_authorizer, method=self.Method, path=self.Path))
+                        'Unable to set RequestModel [{model}] on API method [{method}] for path [{path}] '
+                        'because the related API does not define any Models.'.format(
+                            model=method_model, method=self.Method, path=self.Path))
 
-                if method_authorizer != 'NONE' and not api_authorizers.get(method_authorizer):
+                if not api_models.get(method_model):
                     raise InvalidEventException(
                         self.relative_id,
-                        'Unable to set Authorizer [{authorizer}] on API method [{method}] for path [{path}] because it '
-                        'wasn\'t defined in the API\'s Authorizers.'.format(
-                            authorizer=method_authorizer, method=self.Method, path=self.Path))
+                        'Unable to set RequestModel [{model}] on API method [{method}] for path [{path}] '
+                        'because it wasn\'t defined in the API\'s Models.'.format(
+                            model=method_model, method=self.Method, path=self.Path))
 
-                if method_authorizer == 'NONE' and not api_auth.get('DefaultAuthorizer'):
-                    raise InvalidEventException(
-                        self.relative_id,
-                        'Unable to set Authorizer on API method [{method}] for path [{path}] because \'NONE\' '
-                        'is only a valid value when a DefaultAuthorizer on the API is specified.'.format(
-                            method=self.Method, path=self.Path))
-
-            editor.add_auth_to_method(api=api, path=self.Path, method_name=self.Method, auth=self.Auth)
+                editor.add_request_model_to_method(path=self.Path, method_name=self.Method,
+                                                   request_model=self.RequestModel)
 
         api["DefinitionBody"] = editor.swagger
 
